{% extends base_template %}
{% load i18n %}
{% block content %}
    <div class="form-group">
        <strong>{{ model_options.verbose_name }}:</strong><br/>
        <span id='id_object_name'>{{ object }}</span><br/>
    </div>
    <div class="form-group">
        <p>{% trans 'Are you sure you want to delete this?' %}</p>
        <form role="form" id="delete-form" action="" method="post">
            {% csrf_token %}
            <div class="form-group">
                <button type="submit" class="btn btn-danger">{% trans 'Yes' %}</button>
                <a class="btn btn-primary" href="javascript:window.history.back();">{% trans 'No' %}</a>
            </div>
        </form >
    </div>
{% endblock content %}
